/******************************************************************************
 (c) Copyright 2002-2006, 1060 Research Ltd

 This Software is licensed to You, the licensee, for use under the terms of
 the 1060 Public License v1.0. Please read and agree to the 1060 Public
 License v1.0 [www.1060research.com/license] before using or redistributing
 this software.

 In summary the 1060 Public license has the following conditions.
 A. You may use the Software free of charge provided you agree to the terms
 laid out in the 1060 Public License v1.0
 B. You are only permitted to use the Software with components or applications
 that provide you with OSI Certified Open Source Code [www.opensource.org], or
 for which licensing has been approved by 1060 Research Limited.
 You may write your own software for execution by this Software provided any
 distribution of your software with this Software complies with terms set out
 in section 2 of the 1060 Public License v1.0
 C. You may redistribute the Software provided you comply with the terms of
 the 1060 Public License v1.0 and that no warranty is implied or given.
 D. If you find you are unable to comply with this license you may seek to
 obtain an alternative license from 1060 Research Limited by contacting
 license@1060research.com or by visiting www.1060research.com

 NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 LICENSE V1.0 FOR DETAILS

 THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license

 File:          $RCSfile$
 Version:       $Name$ $Revision$
 Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer0.logging;

import java.net.URL;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import org.netkernel.container.ILogger;
import org.netkernel.container.IMessages;
import org.netkernel.layer0.util.Utils;

/** Implementation of ILogger using a resource bundle to supply messages
 * @author Tony Butterfield
 */
public class KernelLog extends ApplicationLogImpl implements ILogger
{
	private ResourceBundle mResourceBundle;
	
	public KernelLog(URL aConfigURL, ResourceBundle aBundle, LogManager aManager) throws Exception
	{	super(aConfigURL,new BootstrapMessages(aBundle),aManager);
		mResourceBundle = aBundle;
	}

	public void log(int aLevel, Object aCallingObject, String aId, Object... p)
	{	if (shouldLog(aLevel,aCallingObject))
		{	String message = format(aId,p);
			logRaw(aLevel, aCallingObject, message);
		}
	}

	public String getMessage(String aMessageId)
	{	try
		{	return mResourceBundle.getString(aMessageId);
		} catch (MissingResourceException e)
		{	return '!' + aMessageId + '!';
		}		
	}
		
	public String format(String aId, Object... p)
	{	String message = getMessage(aId);
	    StringBuilder buf = new StringBuilder(message.length() + 128);
	    buf.append(message);
	    Utils.replaceParameters(buf, p);
	    return buf.toString();
	}
	
	private static class BootstrapMessages implements IMessages
	{
		private ResourceBundle mResourceBundle;
		
		public BootstrapMessages(ResourceBundle aBundle)
		{	mResourceBundle=aBundle;
		}

		public String format(String aMessageId, Object... p)
		{	String message;
			try
			{	message=mResourceBundle.getString(aMessageId);
			} catch (MissingResourceException e)
			{	message= '!' + aMessageId + '!';
			}	
		    StringBuilder buf = new StringBuilder(message.length() + 128);
		    buf.append(message);
		    Utils.replaceParameters(buf, p);
		    return buf.toString();
		}
	}
}