/******************************************************************************
 * (c) Copyright 2002-2006, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile$
 * Version:       $Name$ $Revision$
 * Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer0.nkf.impl;

import org.netkernel.layer0.nkf.*;
import org.netkernel.request.*;
import org.netkernel.request.impl.MetaImpl;
import org.netkernel.request.impl.ResponseImpl;
import java.lang.reflect.*;

/**
 *
 * @author  tab
 */
public class NKFAsyncRequestHandleImpl implements INKFAsyncRequestHandle, IRequestor, InvocationHandler
{
	private NKFRequestImpl mRequest;
	private NKFContextImpl mContext;
	private IResponse mResult;
	private INKFAsyncRequestListener mListener;
	private IResponse mResponse;
	private boolean mResponseSet;
	
	/** Creates a new instance of NKFSubRequestImpl */
	public NKFAsyncRequestHandleImpl(NKFRequestImpl aRequest, NKFContextImpl aContext)
	{	mRequest = aRequest;
		mContext = aContext;
	}
	
	public Object join() throws NKFException, InterruptedException
	{	return join(0);
	}

	public Object join(long aTimeout) throws NKFException, InterruptedException
	{	synchronized(mRequest)
		{	if (mResult==null)
			{	mRequest.wait(aTimeout);
			}
		}
		if (mResult!=null)
		{	if (mResult.getRepresentation() instanceof Throwable)
			{	NKFException ex=handleException();
				throw ex;
			}
			else
			{	return mResult.getRepresentation();
			}
		}
		else
		{	return null;
		}
	}

	public INKFResponseReadOnly joinForResponse() throws NKFException, InterruptedException
	{	return joinForResponse(0);
	}

	public INKFResponseReadOnly joinForResponse(long aTimeout) throws NKFException, InterruptedException
	{
		synchronized(mRequest)
		{	if (mResult==null)
			{	mRequest.wait(aTimeout);
			}
		}
		if (mResult!=null)
		{	if (mResult.getRepresentation() instanceof Throwable)
			{	NKFException ex=handleException();
				throw ex;
			}
			else
			{	return new NKFResponseReadOnlyImpl(mResult,mRequest,mContext);
			}
		}
		else
		{	return null; // nothing ready yet
		}
	}	
	
	
	
	public void setListener(INKFAsyncRequestListener aListener)
	{	boolean issueCallbackNow=false;
		synchronized(mRequest)
		{	mListener=aListener;
			if (mResult!=null)
			{	issueCallbackNow=true;
			}
		}
		if (issueCallbackNow)
		{	issueCallback();
		}
	}

	public void onResponse(IResponse aResult)
	{	synchronized(mRequest)
		{	mResult=aResult;
			mRequest.notify();
		}
		issueCallback();
	}
	
	private void issueCallback()
	{	try
		{	if (mListener!=null)
			{	// build proxy around context to deal with separate response
				Class[] classes=new Class[]{INKFRequestContext.class, INKFBasicContext.class};
				INKFRequestContext proxy = (INKFRequestContext)Proxy.newProxyInstance(this.getClass().getClassLoader(), classes, this);
				
				if (mResult.getRepresentation() instanceof Throwable)
				{	NKFException nkfe=handleException();
					mListener.receiveException(nkfe,mRequest,proxy);
				}
				else
				{	INKFResponseReadOnly response = new NKFResponseReadOnlyImpl(mResult,mRequest,mContext);
					mListener.receiveResponse(response,proxy);
				}
				// pass response back to kernel if one is given
				mContext.handleAsyncResponse(getResponse());
			}

		}
		catch (Exception e)
		{	mContext.handleAsyncException(e);
		}
	}
	
	/** both errors and exceptions must be passed back to callback so that any
	 * asynchronous "finally" handlers can be executed.
	 */
	private NKFException handleException()
	{	Throwable t=(Throwable)mResult.getRepresentation();
		if (t instanceof Error)
		{	throw (Error)t;
		}
		else
		{	NKFException result = new NKFException(mContext.getKernel().getLogger().format("EX_NKF_SUBREQ"),null,t);
			return result;
		}
	}
	
	/** intercept the setResponse methods and capture response local to this request listener
	 **/
	public Object invoke(Object proxy, Method method, Object[] args) throws Throwable
	{	if (method.getName().equals("setNoResponse"))
		{	mResponse=null;
			mResponseSet = true;
			return null;
		}
		else if (method.getName().equals("createResponseFrom"))
		{	Object arg =args[0];
			mResponseSet=true;
			if (arg instanceof INKFResponseReadOnly)
			{	INKFResponseReadOnly response = (INKFResponseReadOnly)arg;
				NKFResponseImpl resp = new NKFResponseImpl(response,mContext);
				mResponse=resp.getKernelResponse();				
				return resp;
			}
			else
			{	//mContext.declareDependency((IRepresentation)arg);
				NKFResponseImpl resp = new NKFResponseImpl(arg,mContext);
				mResponse=resp.getKernelResponse();
				return resp;
			}
		}
		else
		{	return method.invoke(mContext,args);
		}
	}
	
	
	IResponse getResponse() throws NKFException
	{	if (!mResponseSet)
		{	mResponse = new ResponseImpl(mContext.getThisKernelRequest(),null,MetaImpl.EXPIRED);
		}
		mResponseSet=false;
		return mResponse;
	}
}