/******************************************************************************
 * (c) Copyright 2002-2006, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile$
 * Version:       $Name$ $Revision$
 * Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer0.nkf.impl;

import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.urii.ParsedIdentifierImpl;
import org.netkernel.layer0.urii.SimpleIdentifierImpl;
import org.netkernel.layer0.urii.ValueSpace;
import org.netkernel.layer0.util.OrderedCheapMultiStringMap;
import org.netkernel.request.*;
import org.netkernel.request.impl.*;
import org.netkernel.urii.*;


/**
 * Wrapper around the construction of a sub-request from within an accessor
 * @author  tab
 */
public class NKFRequestImpl implements INKFRequest
{
	protected NKFContextImpl mContext;

	protected int mType = IRequest.VERB_SOURCE;
	protected String mBaseURI;
	protected Class mRepresentationClass = Object.class;
	protected OrderedCheapMultiStringMap mArguments;
	protected ValueSpace mValueSpace;
	protected IRequestScopeLevel mScope;
	protected IResponse mPrimary;
	private LiteralParameterException mParameterRequest;
	protected RequestResponseFieldsWithStickyImpl mHeaders;
	
	/** Creates a new instance of NKFRequestImpl */
	public NKFRequestImpl(NKFContextImpl aContext, String aIdentifier)
	{	mContext=aContext;
		mBaseURI=aIdentifier;
	}
	
	public NKFRequestImpl(NKFContextImpl aContext, String aIdentifier, LiteralParameterException aLPE)
	{	this(aContext,aIdentifier);
		mParameterRequest=aLPE;
	}
	
	/** Advanced internal use only
	 * @param aContext
	 */
	public void setContext(NKFContextImpl aContext)
	{	mContext=aContext;
	}
	
	public boolean isParameterRequest()
	{	return mParameterRequest!=null;
	}
	
	public LiteralParameterException getParameterRequest()
	{	return mParameterRequest;
	}
	
	public IRequest getKernelRequest() throws NKFException
	{	
		IRequestScopeLevel scope=null;
		if (mScope!=null)
		{	scope=mScope;
		}
		else
		{	scope=mContext.getRequestScope();
		}
		
		IRequest parentRequest = mContext.getThisKernelRequestOrNull();
		
		// determine URI
		if (mBaseURI==null) throw createIAE("EX_NKF_REQ_NO_URI");
		
		// construct request
		IIdentifier uri;
		if (mArguments!=null)
		{	uri = new SimpleIdentifierImpl(ParsedIdentifierImpl.parseActiveSyntax(mBaseURI,mArguments).toString());
		}
		else
		{	uri = new SimpleIdentifierImpl(mBaseURI);
		}
		
		if (mValueSpace!=null)
		{	scope = RequestScopeLevelImpl.createDurableScopeLevel(mValueSpace,scope);
			try
			{	mValueSpace.onCommissionSpace(mContext.getKernel());
			} catch (Exception e)
			{	// won't happen
			}
		}
		
		Object[] values=null;
		if (mPrimary!=null)
		{	values = new Object[] { mPrimary };
		}
		
		IRequestResponseFields headers = determineHeaders();
		
		return new RequestImpl(mType,uri,scope,mRepresentationClass,parentRequest,null,headers,values);		
	}

	//determine request headers
	protected IRequestResponseFields determineHeaders()
	{
		RequestResponseFieldsWithStickyImpl headers;
		IRequestResponseFields stickyHeaders=mContext.getStickyRequestHeaders();
		
		if (stickyHeaders.size()>0)
		{	headers = new RequestResponseFieldsWithStickyImpl(stickyHeaders);
			if (mHeaders!=null)
			{	//merge new headers, replacing existing ones
				int size=mHeaders.size();
				for (int i=0; i<size; i++)
				{	headers.put(mHeaders.getKey(i),mHeaders.getValue(i),mHeaders.isSticky(i));
				}
			}
		}
		else if (mHeaders!=null)
		{	headers = mHeaders;
		}
		else
		{	headers = RequestResponseFieldsWithStickyImpl.EMPTY;
		}
		return headers;
	}
	

	public String getIdentifier() throws NKFException
	{	IIdentifier uri;
		if (mArguments!=null)
		{	uri = new SimpleIdentifierImpl(ParsedIdentifierImpl.parseActiveSyntax(mBaseURI,mArguments).toString());
		}
		else
		{	uri = new SimpleIdentifierImpl(mBaseURI);
		}
		return uri.toString();
	}	
	
	protected static final Object NO_REP=new Boolean(false);
	
	protected void addArgument(String aName, String aURI, Object aRepresentation, IResponseMeta aMeta)
	{	if (aName!=null)
		{	for (int i=aName.length()-1; i>=0; i--)
				{	char c = aName.charAt(i);
				if (!Character.isLetterOrDigit(c) && c!='-')
				{	throw createIAE("EX_NKF_REQ_ARG_CHAR",aName);
				}
			}
			
			if (mArguments==null)
			{	mArguments = new OrderedCheapMultiStringMap(8);
			}
			mArguments.put(aName, aURI);
		}
		if (aRepresentation!=NO_REP)
		{	if (mValueSpace==null)
			{	mValueSpace = new ValueSpace(4);
			}
			mValueSpace.put(aURI,aRepresentation,aMeta);
		}		
	}
	
	public void addArgument(String aName, String aURI) throws NKFException
	{	if (aName==null) throw createIAE("EX_NKF_REQ_ARG_NAME");
		if (aURI==null) throw createIAE("EX_NKF_REQ_ARG_URI");
		try
		{	String uri = mContext.resolveIdentifier(aURI,true);
			addArgument(aName, uri, NO_REP, null);
		}
		catch (LiteralParameterException e)
		{	Object parameterRepresentation = e.getRepresentation();
			this.addArgumentByValue(aName, parameterRepresentation);
		}
	}
	
	public void addArgumentAsAbsolute(String aName, String aURI) throws NKFException
	{	if (aName==null) throw createIAE("EX_NKF_REQ_ARG_NAME");
		if (aURI==null) throw createIAE("EX_NKF_REQ_ARG_URI");
		try
		{	String uri = mContext.resolveIdentifier(aURI,false);
			addArgument(aName, uri, NO_REP, null);
		}
		catch (LiteralParameterException e)
		{	Object parameterRepresentation = e.getRepresentation();
			this.addArgumentByValue(aName, parameterRepresentation);
		}
	}
	
	public void addArgumentByValue(String aName, Object aRepresentation)
	{	if (aName==null) throw createIAE("EX_NKF_REQ_ARG_NAME");
		String unique = createLiteralURI(aName);
		addArgument(aName, unique, aRepresentation,null);
	}

	public void addPrimaryArgument(Object aValue)
	{	mPrimary = new ResponseImpl(null,aValue,MetaImpl.EXPIRED);
	}
	
	public void addPrimaryArgumentFromResponse(INKFResponseReadOnly aResponse)
	{	if (aResponse==null) throw createIAE("EX_NKF_REQ_ARG_RESPONSE");
		if (aResponse instanceof NKFResponseImpl)
		{	mPrimary = ((NKFResponseImpl)aResponse).getKernelResponse();
		}
		else if (aResponse instanceof NKFResponseReadOnlyImpl)
		{	mPrimary = ((NKFResponseReadOnlyImpl)aResponse).getKernelResponse();
		}
	}
	
	void addPrimaryArgumentWithMetaData(Object aRepresentation, IResponseMeta aMeta)
	{	mPrimary = new ResponseImpl(null,aRepresentation,aMeta);
	}	

	public void addArgumentFromResponse(String aName, INKFResponseReadOnly aResponse)
	{	if (aName==null) throw createIAE("EX_NKF_REQ_ARG_NAME");
		if (aResponse==null) throw createIAE("EX_NKF_REQ_ARG_RESPONSE");
		String unique = createLiteralURI(aName);
		IResponse kernelResponse;
		if (aResponse instanceof NKFResponseReadOnlyImpl)
		{	NKFResponseReadOnlyImpl response = (NKFResponseReadOnlyImpl)aResponse;
			kernelResponse=response.getKernelResponse();
		}
		else
		{	NKFResponseImpl response = (NKFResponseImpl)aResponse;
			kernelResponse=response.getKernelResponse();
		}
		addArgument(aName,unique,kernelResponse.getRepresentation(),kernelResponse.getMeta());
	}

	public void setRepresentationClass(Class aRepresentationClass)
	{	if (aRepresentationClass==null) aRepresentationClass=Object.class;
		mRepresentationClass = aRepresentationClass;
	}
	
	public Class getRepresentationClass()
	{	return mRepresentationClass;
	}
	
	public void setVerb(int aType)
	{	mType = aType;
	}
	
	public void setPriority(int aPriority)
	{	if (aPriority>INKFRequestReadOnly.PRIORITY_MAX || aPriority<INKFRequestReadOnly.PRIORITY_MIN)
		{	throw createIAE("EX_NKF_REQ_PRIORITY");
		}
		else
		{	setHeader(IRequest.HEADER_PRIORITY, aPriority);
		}
	}
	
	public void setRequestScope(IRequestScopeLevel aRequestScope)
	{	
		mScope=aRequestScope;
	}
	
	public void injectRequestScope(ISpace aSpace)
	{	IRequestScopeLevel parentScope;
		if (mScope==null)
		{	parentScope=mContext.getRequestScope();
		}
		else
		{	parentScope = mScope;
		}
		mScope = RequestScopeLevelImpl.createNonDurableScopeLevel(aSpace, parentScope);
	}
	
	public void injectDurableRequestScope(ISpace aSpace)
	{	IRequestScopeLevel parentScope;
		if (mScope==null)
		{	parentScope=mContext.getRequestScope();
		}
		else
		{	parentScope = mScope;
		}
		mScope = RequestScopeLevelImpl.createDurableScopeLevel(aSpace, parentScope);
	}

	private String createLiteralURI(String aName)
	{	
		int i=0;
		if (mArguments!=null && mArguments.get(aName)!=null)
		{	for (int j=0; j<mArguments.size(); j++)
			{	if (mArguments.getKey(j).equals(aName))
				{	i++;
				}
			}
		}
		
		StringBuffer sb=new StringBuffer(aName.length()+6);
		sb.append("var:");
		sb.append(aName);
		if (i>0)
		{	sb.append(i);
		}
		return sb.toString();
	}
	
	protected IllegalArgumentException createIAE(String aKey)
	{	return new IllegalArgumentException(mContext.getKernel().getLogger().format(aKey));
	}
	
	protected IllegalArgumentException createIAE(String aKey, String aParameter)
	{	return new IllegalArgumentException(mContext.getKernel().getLogger().format(aKey,aParameter));
	}

	public void setHeader(String aName, Object aRepresentation)
	{	setHeader(aName,aRepresentation,false);
	}
		
	public void setHeader(String aName, Object aRepresentation, boolean aSticky)
	{	if (mHeaders==null)
		{	mHeaders=new RequestResponseFieldsWithStickyImpl(4);
		}
		mHeaders.put(aName,aRepresentation,aSticky);
	}
	
	public String toString()
	{	try
		{	return RequestFactory.toString(getKernelRequest());
		}
		catch (NKFException e)
		{	return super.toString();
		}
	}
}