/******************************************************************************
 * (c) Copyright 2002-2006, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile$
 * Version:       $Name$ $Revision$
 * Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer0.nkf.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.request.IRequestResponseFields;
import org.netkernel.request.IResponse;
import org.netkernel.request.IResponseMeta;


public class NKFResponseReadOnlyImpl<R> implements INKFResponseReadOnly<R>
{
	private IResponse mResponse;
	private INKFRequest mRequest;
	private NKFContextImpl mContext;
	
	public NKFResponseReadOnlyImpl(IResponse aResponse, INKFRequest aRequest, NKFContextImpl aContext)
	{	mResponse=aResponse;
		mRequest=aRequest;
		mContext=aContext;
	}
	
	IRequestResponseFields getUserMeta()
	{	return mResponse.getMeta().getUserMetaData();
	}
	
	public IResponse getKernelResponse()
	{	return mResponse;
	}
	
	public INKFRequest getRequest()
	{	return mRequest;
	}

	public R getRepresentation()
	{	return (R)mResponse.getRepresentation();
	}
	
	public IResponseMeta getMeta()
	{	return mResponse.getMeta();
	}

	public boolean isExpired()
	{	return mResponse.getMeta().getExpiry().isExpired(System.currentTimeMillis());
	}

	public Iterator getHeaderNames()
	{	IRequestResponseFields userMeta =mResponse.getMeta().getUserMetaData(); 
		int size=userMeta.size();
		List result = new ArrayList(size);
		for (int i=0; i<size; i++)
		{	result.add(userMeta.getKey(i));
		}
		return result.iterator();
	}

	public Object getHeader(String aKey) throws NKFException
	{	Object result = mResponse.getMeta().getUserMetaData().getValue(aKey);
		if (result==null)
		{	String id=mContext.getKernel().getLogger().format("EX_NKF_METADATA_KEY");
			throw new NKFException(id,aKey);
		}
		return result;
	}
	
	public IRequestResponseFields getHeaders()
	{	return mResponse.getMeta().getUserMetaData();
	}
	
	public boolean hasHeader(String aKey)
	{	return mResponse.getMeta().getUserMetaData().getValue(aKey)!=null;
	}

	public Object getHeaderAs(String aKey, Class aRepresentationClass) throws NKFException
	{
		Object representation=getHeader(aKey);
		if (!aRepresentationClass.isAssignableFrom(representation.getClass()))
		{	representation=mContext.transrept(representation,aRepresentationClass);
		}
		return representation;
	}	

	public String getMimeType()
	{	
		String result=(String)mResponse.getMeta().getUserMetaData().getValue(IResponseMeta.KEY_MIMETYPE);
		if (result==null)
		{	result=MIME_UNKNOWN;
		}
		return result;
	}

}